#include <Wire.h>

/*
   Texas Northern Model Railroad Club
   Yard C west end contorl Sketch.
   Controls access to Yard C from Yard A track 11, and Mountain mainline.


*/


#include <arduino.h>

#define TURNOUTCOUNT 7
#define STATEBASE 65

byte current_state; // state is 1 to 8, 8 track yard.


//saved value for relay control.
int turnout_pins[TURNOUTCOUNT] = {4, 5, 6, 7, 8, 9, 10}; //
int turnout_value[TURNOUTCOUNT] = {LOW, LOW, LOW, LOW, LOW, LOW, LOW};

int panel_id[3] = {50, 51, 52}; // allow for future multiple panels
int active_slaves = 1;

byte messageArray[32];


void setup() {

  Serial.begin(9600);
  Serial.println("TXNMRRC Yard 44/46 entrance Master with I2C connections to slaves. V0.0.3");

  Wire.begin();

  // set switch relays to output
  int x;
  for (x = 0; x < TURNOUTCOUNT; x++) {
    pinMode(turnout_pins[x], OUTPUT);
    turnout_value[x] = LOW;
    digitalWrite(turnout_pins[x], turnout_value[x]);
  };


  // set default state as mainline.
  current_state = STATEBASE;


  set_state (STATEBASE);

}
//--------------------------------------------LOOP--------------------------------
void loop() {
  unsigned long Ltime;
  Ltime = millis();
  Serial.print("[");
  Serial.print(Ltime);
  Serial.print("] ");
  // poll slaves for their current state.

  // test routine to test turnout motor connections
  //byte remote_state; // state read from remote modules.

  //	remote_state = 99; // in case no reply
  for (int x = 0; x < active_slaves; x++)  {
    messageArray[0] = 99;
    Wire.requestFrom(panel_id[x], 1);    // request 2 bytes from slave device #2
    Serial.print("reading from slave #");
    Serial.println(panel_id[x]);
    int v;
    v = 0;
    while (Wire.available())   // slave may send less than requested
    {
      char c;
      c = Wire.read();
      messageArray[v]  = c;
      v++;
      Serial.print("***char read: ");
      Serial.println(c);         // print the character
    }
    if (messageArray[0] > (STATEBASE + 9)) {
      Serial.print("Idle slave ");
      Serial.print(panel_id[x]);
      Serial.print(", state = ");
      Serial.println(current_state);         // print the character
      messageArray[0] = 99;
    }

    if (messageArray[0] < 90) {
      //   messageArray[0] -= STATEBASE;
      if (messageArray[0] != current_state)
      {
        Serial.print("update state to: ");
        current_state = messageArray[0];
        Serial.println(current_state);         // print the character
        set_state(current_state);
      }
    }
  }
  Serial.println("--------------");
  delay(1000); // pause between seweeps.
}

// -------------------------------------------SET STATE ---------------------------
// ******* function to update slave state and set turnout motor relays to new values.
void set_state(int state)
{
  Serial.print("into set state ");
  Serial.println(state);
  // set relay status
  switch (state) {
    default:
      Serial.print("default, check slave!  ");
      Serial.println(state);

    case STATEBASE + 0:
      // align switches for mainline running.
      Serial.println("set state - 1");
      turnout_value[0] = LOW;
      turnout_value[1] = LOW;
      turnout_value[2] = LOW;
      turnout_value[3] = LOW;
      turnout_value[4] = LOW;
      turnout_value[5] = LOW;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together
    case STATEBASE + 1:
      // align switches for mainline running.
      Serial.println("set state -- 2");
      turnout_value[0] = LOW;
      turnout_value[1] = LOW;
      turnout_value[2] = HIGH;
      turnout_value[3] = LOW;
      turnout_value[4] = LOW;
      turnout_value[5] = LOW;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together

    case STATEBASE + 2:
      // align switches for mainline running.
      Serial.println("set state --- 3");
      turnout_value[0] = LOW;
      turnout_value[1] = HIGH;
      turnout_value[2] = LOW;
      turnout_value[3] = LOW;
      turnout_value[4] = LOW;
      turnout_value[5] = LOW;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together

    case STATEBASE + 3:
      // align switches for mainline running.
      Serial.println("set state ---- 4");
      turnout_value[0] = LOW;
      turnout_value[1] = HIGH;
      turnout_value[2] = LOW;
      turnout_value[3] = HIGH;
      turnout_value[4] = LOW;
      turnout_value[5] = LOW;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together

    case STATEBASE + 4:
      // align switches for mainline running.
      Serial.println("set state ----- 5");
      turnout_value[0] = HIGH;
      turnout_value[1] = LOW;
      turnout_value[2] = LOW;
      turnout_value[3] = LOW;
      turnout_value[4] = LOW;
      turnout_value[5] = LOW;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together

    case STATEBASE + 5:
      // align switches for mainline running.
      Serial.println("set state ------ 6");
      turnout_value[0] = HIGH;
      turnout_value[1] = LOW;
      turnout_value[2] = LOW;
      turnout_value[3] = LOW;
      turnout_value[4] = LOW;
      turnout_value[5] = HIGH;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together

    case STATEBASE + 6:
      // align switches for mainline running.
      Serial.println("set state ------- 7");
      turnout_value[0] = HIGH;
      turnout_value[1] = LOW;
      turnout_value[2] = LOW;
      turnout_value[3] = LOW;
      turnout_value[4] = HIGH;
      turnout_value[5] = LOW;
      turnout_value[6] = LOW;
      break; // break to avoid running two states together

    case STATEBASE + 7:
      // align switches for mainline running.
      Serial.println("set state -------- 8");
      turnout_value[0] = HIGH;
      turnout_value[1] = LOW;
      turnout_value[2] = LOW;
      turnout_value[3] = LOW;
      turnout_value[4] = HIGH;
      turnout_value[5] = LOW;
      turnout_value[6] = HIGH;
      break; // break to avoid running two states together

    case STATEBASE + 8:
      // align switches for mainline running.
      Serial.println("set state -------- 9 - invalid");
      turnout_value[0] = HIGH;
      turnout_value[1] = HIGH;
      turnout_value[2] = HIGH;
      turnout_value[3] = HIGH;
      turnout_value[4] = HIGH;
      turnout_value[5] = HIGH;
      turnout_value[6] = HIGH;

      break; // break to avoid running two states together


  } // set switch status

  // set control pin values, this throws or clears each switch relay.
  for (int x = 0; x < TURNOUTCOUNT; x++) {
    digitalWrite(turnout_pins[x], turnout_value[x]);
  };
  for (int y = 0; y < active_slaves; y++)
  {

    Serial.print("sending to slave #");
    Serial.println(panel_id[y]);
    Wire.beginTransmission(panel_id[y]); // transmit to device #1
    Wire.write(state);              // sends one byte
    Wire.endTransmission();    // stop transmitting
  }

  Serial.println();
  Serial.println("pause");

  delay (5000); // 5 seconds - to allow turnout motors to cycle completely between settings.
 
  Serial.println("run after state update");
} // end set state()
